<?xml version="1.0"?>
<xsl:stylesheet version="2.0" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema">
	
<xsl:output method="html" version="4.0" encoding="UTF-8" indent="yes"/>

<xsl:param name="duration" select="P14D"/>
<xsl:param name="showResource"/>
<xsl:param name="withProjectImage"/>
<xsl:param name="withProjectStamp"/>
<xsl:param name="dateFormatter" select="'[D01].[M01].[Y]'"/>

<xsl:template name="formatDuration">
 	<xsl:param name="duration"/>

	<xsl:value-of select="$duration/@amount" />
	<xsl:text> </xsl:text>
	<xsl:if test="$duration/@amount=1">
		<xsl:choose>
			<xsl:when test="$duration/@unit = 'minutes'">Minute</xsl:when>
			<xsl:when test="$duration/@unit = 'hours'">Stunde</xsl:when>
			<xsl:when test="$duration/@unit = 'days'">Tag</xsl:when>
			<xsl:when test="$duration/@unit = 'weeks'">Woche</xsl:when>
			<xsl:when test="$duration/@unit = 'months'">Monat</xsl:when>
			<xsl:when test="$duration/@unit = 'quarters'">Quartal</xsl:when>
			<xsl:when test="$duration/@unit = 'years'">Jahr</xsl:when>
		</xsl:choose>
	</xsl:if>
	<xsl:if test="$duration/@amount!=1">
		<xsl:choose>
			<xsl:when test="$duration/@unit = 'minutes'">Minuten</xsl:when>
			<xsl:when test="$duration/@unit = 'hours'">Stunden</xsl:when>
			<xsl:when test="$duration/@unit = 'days'">Tage</xsl:when>
			<xsl:when test="$duration/@unit = 'weeks'">Wochen</xsl:when>
			<xsl:when test="$duration/@unit = 'months'">Monate</xsl:when>
			<xsl:when test="$duration/@unit = 'quarters'">Quartale</xsl:when>
			<xsl:when test="$duration/@unit = 'years'">Jahre</xsl:when>
		</xsl:choose>
	</xsl:if>
</xsl:template>

<xsl:template match="Project">
	<html>
	<head>
	<meta http-equiv="content-type" content="text/html; charset=utf-8" />
	<link rel="stylesheet" type="text/css" href="rsc/styles.css" />
	</head>
	<body>
	
	<!-- Standard Report Header for Merlin 2 Reports: START -->
	<xsl:if test="$withProjectStamp = 1">
		<div id="projectstamp">
		  <table class="report">
		  <tr>
			<th colspan="2">Projektstempel</th></tr>
			<tr class="r1">
				<td class="title">Projektname</td>
				<td class="projectname"><xsl:value-of select="title"/></td>
			</tr>
			<tr class="r0">
				<td class="title">Geplant: Start-Ende</td>
				<td>
					<xsl:value-of select="format-dateTime(plannedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(plannedEndDate, $dateFormatter)"/> 
				</td>
			</tr>
			<tr class="r1">
				<td class="title">Erwartet: Start-Ende</td>
				<td>
					<xsl:value-of select="format-dateTime(expectedStartDate, $dateFormatter)"/> - 
					<xsl:value-of select="format-dateTime(expectedEndDate, $dateFormatter)"/>
				</td>
			</tr>
			<xsl:if test="description != ''">
				<tr class="r0">
					<td class="title">Ziel</td>
					<td><xsl:value-of select="description"/></td>
				</tr>
			</xsl:if>	
		</table>
		</div>
	</xsl:if>
	<xsl:if test="$withProjectImage = 1">
		<div id="logo">
			<img src="rsc/logo.png" />
		</div>
	</xsl:if> 
	<!-- Standard Report Header for Merlin 2 Reports: END -->
	
	<div id="break" />
	
	<xsl:variable name="now" select="xs:dateTime(current-date())"/>
	<xsl:variable name="maxDate" select="$now + xs:dayTimeDuration($duration)"/>

	<xsl:if test="$showResource = ''">
		<h1>Beschreibende Projekt- und Vorgangs-Informationen</h1>
	</xsl:if> 

	<xsl:if test="$showResource !='' ">
		<h1>Beschreibende Projekt- und Vorgangs-Informationen für <xsl:value-of select="$showResource"/></h1>
	</xsl:if> 

    <div id="nextduedates">

	<table class="report">
	<tr>
	<th>MS</th>
	<th>Titel</th>
	<th>Gruppe</th>
	<th>Ressourcen</th>
	<th>Informationen</th>
	</tr>
	
	<xsl:if test="$showResource = '' ">
		<xsl:for-each select="//Activity">
		<xsl:sort select="expectedStartDate"/>
		<tr class="r{position() mod 2}">
			<td valign="top">
				<xsl:if test="isMilestone">◆</xsl:if>
			</td>
			<td valign="top"><xsl:value-of select="title"/></td>
			<td valign="top"><xsl:value-of select="../title"/></td>			
			<td valign="top">
				<xsl:for-each select="./Assignment">
					<xsl:value-of select="title"/><br />
				</xsl:for-each>
			</td>
			<td valign="top" width="75%"><pre><xsl:value-of select="description"/></pre></td>
		</tr>
		</xsl:for-each>
	</xsl:if> 


	<xsl:if test="$showResource != '' ">
		<xsl:for-each select="//Activity [./Assignment/title=$showResource]">
		<xsl:sort select="expectedStartDate"/>
		<tr class="r{position() mod 2}">
			<td valign="top">
				<xsl:if test="isMilestone">◆</xsl:if>
			</td>
			<td valign="top"><xsl:value-of select="title"/></td>
			<td valign="top"><xsl:value-of select="../title"/></td>			
			<td valign="top">
				<xsl:for-each select="./Assignment">
					<xsl:value-of select="title"/><br />
				</xsl:for-each>
			</td>
			<td valign="top" width="75%"><pre><xsl:value-of select="description"/></pre></td>
		</tr>
		</xsl:for-each>
	</xsl:if> 

	
	</table>
	</div>
		
	<div id="copyright">
		Generated by Merlin — <a href="http://www.merlin2.net/" target="_blank">www.merlin2.net</a>
	</div>

  </body>
  </html>
 
</xsl:template>

</xsl:stylesheet>