class MERMilestoneTrend(PWReportPage):
	def init(self):
		super(MERMilestoneTrend, self).init()
		self.project		= None
		self.plannedColor 	= None
		self.expectedColor  = None
		self.iterMilestone	= None
		self.iterIndex		= 0
		return self

	def milestones(self):
		activities = self.project.linkedSubActivitiesDeepIncludeMe()
		predicate = NSPredicate.predicateWithFormat_("isMilestone = TRUE AND isPrivate = FALSE AND isAssignment = NO AND mayBeSeenByCurrentUser = YES")
		milestones = activities.filteredArrayUsingPredicate_(predicate)
		startDateDescriptor = NSSortDescriptor.alloc().initWithKey_ascending_("plannedStartDate", True)
		return milestones.sortedArrayUsingDescriptors_([startDateDescriptor])

	def xTicks(self):
		xTicks = "\"xTicks\": [ "
		milestones = self.milestones()
		
		for i in range(len(milestones)):
			milestone = milestones[i]
			if i<len(milestones)-1:
				separator = ","
			else:
				separator = ""
			if self.withMilestoneLabels:
				title = milestone.title().replace("\"", "\\\"")
				xTicks += "{v:%d, label:\"%s\"}%s\n" % (i, title, separator)
			else:
				xTicks += "{v:%d, label:\"%s\"}%s\n" % (i, "", separator)
		xTicks = xTicks + "]"
		return xTicks
		
	
	def yAxisAndTicks(self):
		divider = 9
		minTime = NSDate.distantFuture().timeIntervalSince1970()
		maxTime = 0.0
		milestones = self.milestones()
		for i in range(len(milestones)):
			activity = milestones[i]
			time = activity.plannedStartDate().timeIntervalSince1970()
			if time < minTime:
				minTime = time
			if time > maxTime:
				maxTime = time
			time = activity.expectedStartDate().timeIntervalSince1970()
			if time < minTime:
				minTime = time
			if time > maxTime:
				maxTime = time
		formatter = NSDateFormatter.alloc().init()
		formatter.setFormatterBehavior_(NSDateFormatterBehavior10_4)
		formatter.setDateStyle_(NSDateFormatterShortStyle)
		formatter.setTimeStyle_(NSDateFormatterNoStyle)
		timeSlice = (maxTime-minTime)/divider
		yAxisAndTicks = "\"yAxis\": [ %f, %f],\n"%(minTime, maxTime)
		yAxisAndTicks += "\"yTicks\": [ "
		t = minTime
		for i in range(divider+1):
			dateString = formatter.stringFromDate_(NSDate.dateWithTimeIntervalSince1970_(t))
			if(i<divider):
				separator = ","
			else:
				separator = ""
			yAxisAndTicks += "{v:%f, label:\"%s\"}%s\n"%(t, dateString, separator)
			t+=timeSlice
		dateString = formatter.stringFromDate_(NSDate.dateWithTimeIntervalSince1970_(t))
		yAxisAndTicks += "]\n"
		return yAxisAndTicks
	
	def dataSetStringForKey_(self, key):
		dataSetString = "\"%s\",[" % (key)
		milestones = self.milestones()
		for i in range(len(milestones)):
			activity = milestones[i]
			if i<len(milestones)-1:
				separator = ","
			else:
				separator = ""
			dataSetString += "[%d, %f]%s" % (i, activity.valueForKey_(key).timeIntervalSince1970(), separator)
		dataSetString += "]"
		return dataSetString
			
	def plannedDataSetString(self):
		return self.dataSetStringForKey_("plannedStartDate")
		
	def expectedDataSetString(self):
		return self.dataSetStringForKey_("expectedStartDate")
		
	def dateFormat(self):
		return "medium/short"

	def showObjective(self):
		return self.project.itemDescription()

	def rowClass(self):
		if self.iterIndex%2:
			return "even"

	def plannedColorString(self):
		return self.plannedColor.webColor()

	def expectedColorString(self):
		return self.expectedColor.webColor()

